#ifndef __COMMANDS_H
#define __COMMANDS_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "callbacks.h"

#define MAX_CMD_TEXT_LENGTH 100

enum Message{
	BEGIN_CHAR = 0xBE,
	END_CHAR   = 0xED
};

/*
 * Enumeration of the data types that a callback function may use
 * doubleType should get added here at some point
 */
enum DataType
{
	floatType,            // 00
	intType,              // 01
	stringType            // 02
};

/*
 * Message type IDs used to know what kind of message we are dealing with
 * Enumeration used to index the MessageTypes array in commands.c
 *
 * DO NOT change this enum or you will break backwards compatibility.
 * DO NOT change this enum without also updating the "MessageTypes" array
 * in commands.c to match.
 */
enum MessageTypeID{
	DEBUG_ID,             // 00
	PACKETLOG_ID,         // 01
	GETPACKETLOGS_ID,     // 02
	UPDATE_ID,            // 03
	BEGINUPDATE_ID,       // 04
	LOG_ID,               // 05
	RESPONSE_ID,          // 06
	SETPARAM_ID,          // 07 - Setting controller parameters. Example: PID constants
	GETPARAM_ID,          // 08 - Getting controller parameters. Example: PID constants
	RESPPARAM_ID,         // 09 - Responding with controller parameters. Example: PID constants
	MAX_TYPE_ID           // 10 - Just used to keep track of the size
};

/*
 * Controller ID used to know which controller the message is referencing
 */
enum ControllerID{
	ROLL_ID,              // 00 - Roll PID
	PITCH_ID,             // 01 - Pitch PID
	YAW_ID,               // 02 - Yaw PID
	ROLL_RATE_ID,         // 03 - Roll rate PID
	PITCH_RATE_ID,        // 04 - Pitch rate PID
	YAW_RATE_ID,          // 05 - Yaw rate PID
	LOCAL_X_ID,           // 06 - Local X PID
	LOCAL_Y_ID,           // 07 - Local Y PID
	ALT_ID,               // 08 - Altitude PID
	MAX_CONTROLLER_ID,    // 09 - Just used to keep track of the size
};

/*
 * Enumeration of controller parameters
 */
enum ControlParamID{
	KP_ID,                 // 00 - P constant
	KI_ID,                 // 01 - I constant
	KD_ID,                 // 02 - D constant
	SP_ID,                 // 03 - Setpoint value
	MAX_CONTROL_PARAM_ID,  // 04 - Just used to keep track of the size
};

/*
 * Message type struct used to keep track of the callback function
 * pointers located in commands.c
 */
struct MessageType{
	char cmdText[MAX_CMD_TEXT_LENGTH];
	char cmdDataType;
	command_cb * functionPtr;
};

/* Defined in commands.c */
extern struct MessageType MessageTypes[MAX_TYPE_ID];

/* Legacy functions - no idea what uses these. Please do not delete. */
float getFloat(unsigned char* str, int pos);
int getInt(unsigned char* str, int pos);
/* end legacy crap */

#endif /* __COMMANDS_H */
